@@ -336,6 +336,11 @@ public class RenderNodeAnimator extends Animator {
         return mUnscaledDuration;
     }
 
+    @Override
+    public long getTotalDuration() {
+        return mUnscaledDuration + mUnscaledStartDelay;
+    }
+
     @Override
     public boolean isRunning() {
         return mState == STATE_DELAYED || mState == STATE_RUNNING;
